#!/usr/bin/env python
#
# Lightsaber rotoscoping scripts
# Creates the working directory and project file (project_info)
# Decodes movie file into individual frames
#
# Matthew Beckler - matthew at mbeckler dot org
# http://www.mbeckler.org/lightsabers/
# Version 1.01 - May 17, 2011

import sys, os, shutil, glob
from subprocess import *
from math import ceil, sqrt
from PIL import Image

if len(sys.argv) < 2:
    print "Usage: %s videoFile" % sys.argv[0]
    sys.exit(1)

print "Parsing command line options ..."
filename = sys.argv[1]
basename, extension = os.path.splitext(filename)
basename = os.path.basename(basename)
newfilename = basename + extension
proj_filename = "project_info"
if not os.path.exists(filename):
    print "File '%s' does not exist" % filename
    sys.exit(1)
if os.path.exists(basename):
    shutil.rmtree(basename)

print "Creating work directory '%s'" % basename
os.mkdir(basename)
shutil.copy(filename, basename)
orig_filename = basename + "_original" + extension
os.rename(filename, orig_filename)
os.chdir(basename)

print "Extracting frames from movie ..."
p = Popen(["mplayer", "-ao", "pcm:waveheader:file=audio.wav", "-vo", "png:z=1", newfilename], stdout=PIPE, stderr=PIPE)
os.waitpid(p.pid, 0)

os.remove(newfilename)

print "Writing project info file ..."
output = Popen(["identify", "-format", "\"%w %h\"", "00000001.png"], stdout=PIPE).communicate()[0].rstrip().strip("\"").split()
width = int(output[0])
height = int(output[1])

pngList = glob.glob("*.png")
num_frames = len(pngList)
if num_frames == 0:
    print "num_frames detected to be zero; something went very wrong"
    sys.exit(1)

buffer_size = 50 # pixels
# should be large enough that blurs don't encroach on adjacent frames, but not
# so large that it makes the mosaic unwieldy - 50 is a good number

proj_file = open(proj_filename, "w")
proj_file.write("%s # basename\n" % basename)
proj_file.write("%s # image width\n" % width)
proj_file.write("%s # image height\n" % height)
proj_file.write("%s # number of frames\n" % num_frames)
proj_file.write("%s # buffer size\n" % buffer_size)
proj_file.close()
print "Basename: '%s'" % basename
print "Image width: %d" % width
print "Image height: %d" % height
print "Number of frames: %d" % num_frames
print "Buffer size: %d" % buffer_size


# this is the code that actually creates the mosaic
print "Creating mosaic ..."
# we use the square root to make the mosaic as square as possible
num_cols = int(ceil(sqrt(num_frames)))
num_rows = int(ceil(num_frames / (1.0 * num_cols)))
print "Number of rows: %d" % num_rows
print "Number of columns: %d" % num_cols

# these are the total dimensions of the mosaic
mosaic_width = num_cols * ( (2 * buffer_size) + width )
mosaic_height = num_rows * ( (2 * buffer_size) + height )

# creates the mosaic in memory, defaults to black
mosaic = Image.new("RGBA", (mosaic_width, mosaic_height), "#000000")

print "Adding images ",
sys.stdout.flush()
i = 1
for y in range(0, num_rows):
    for x in range(0, num_cols):
        # The mosaic will not be fully filled unless the number of files
        # is a perfect square. Normally, when we run out of images to use
        # (when $i > $numberOfFiles), we stop copying, and just wait until
        # the loops finish.
        if i <= num_frames:
            img_filename = "%08d.png" % i
            # load the current image from file
            img = Image.open(img_filename)
            # these are the coordinate to paste the loaded image
            target_x = (x * ( (2 * buffer_size) + width )) + buffer_size
            target_y = (y * ( (2 * buffer_size) + height )) + buffer_size
            # paste in the image
            mosaic.paste(img, (target_x, target_y))
            sys.stdout.write(".")
            sys.stdout.flush()
            i = i + 1
        else:
            break
print " done"
print "Writing mosaic file ...",
sys.stdout.flush()
mosaic.save("mosaic.png")
print "done"


print "Removing individual frames ...",
sys.stdout.flush()
for f in pngList:
    os.remove(f)
print "done"
print ""


print "All finished!"
print "Now, just edit the mosaic file \"mosaic.png\" in \"%s/\"." % basename
print "You can also edit \"audio.wav\" to add sound effects or dramatic music."
print "When you're finished, run this from the current directory to create the output movie:"
print "./finish_project.py %s" % basename
